@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 
 # JinjaFx - Jinja2 Templating Tool
-# Copyright (c) 2020-2021 Chris Mason <chris@netnix.org>
+# Copyright (c) 2020-2022 Chris Mason <chris@netnix.org>
 #
 # Permission to use, copy, modify, and distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -15,9 +15,9 @@
 # ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 # OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-import sys, os, jinja2, yaml, argparse, re, copy, getpass, traceback
+import sys, os, jinja2, yaml, argparse, re, copy, getpass, datetime, pytz, traceback
 
-__version__ = '1.8.0'
+__version__ = '1.8.5'
 jinja2_filters = []
 
 def import_filters(errc = 0):
@@ -65,9 +65,9 @@ def main():
   try:
     if '-q' not in sys.argv:
       print('JinjaFx v' + __version__ + ' - Jinja2 Templating Tool')
-      print('Copyright (c) 2020-2021 Chris Mason <chris@netnix.org>\n')
+      print('Copyright (c) 2020-2022 Chris Mason <chris@netnix.org>\n')
 
-    jinjafx_usage = '(-t <template.j2> [-d <data.csv>] | -dt <dt.yml>) [-g <vars.yml>] [-o <output file>] [-od <output dir>] [-m] [-q]'
+    jinjafx_usage = '(-t <template.j2> [-d <data.csv>] | -dt <dt.yml>) [-g <vars.yml>] [-ed <exts dir>] [-o <output file>] [-od <output dir>] [-m] [-q]'
 
     parser = __ArgumentParser(add_help=False, usage='%(prog)s ' + jinjafx_usage)
     group_ex = parser.add_mutually_exclusive_group(required=True)
@@ -75,6 +75,7 @@ def main():
     group_ex.add_argument('-t', metavar='<template.j2>', type=argparse.FileType('r'))
     parser.add_argument('-d', metavar='<data.csv>', type=argparse.FileType('r'))
     parser.add_argument('-g', metavar='<vars.yml>', type=argparse.FileType('r'), action='append')
+    parser.add_argument('-ed', metavar='<exts dir>', type=str, action='append', default=[])
     parser.add_argument('-o', metavar='<output file>', type=str)
     parser.add_argument('-od', metavar='<output dir>', type=str)
     parser.add_argument('-m', action='store_true')
@@ -216,7 +217,8 @@ def main():
     if import_filters() > 0:
       print()
 
-    outputs = JinjaFx().jinjafx(args.t, data, gvars, args.o)
+    args.ed = [os.getcwd(), os.getenv('HOME') + '/.jinjafx'] + args.ed
+    outputs = JinjaFx().jinjafx(args.t, data, gvars, args.o, args.ed)
     ocount = 0
 
     if args.od is not None:
@@ -282,10 +284,11 @@ class __ArgumentParser(argparse.ArgumentParser):
 
 
 class JinjaFx():
-  def jinjafx(self, template, data, gvars, output):
+  def jinjafx(self, template, data, gvars, output, exts_dirs=[]):
     self.__g_datarows = []
     self.__g_dict = {}
     self.__g_row = 0 
+    self.__g_vars = {}
     self.__g_warnings = []
 
     outputs = {}
@@ -457,9 +460,6 @@ class JinjaFx():
     if 'jinja2_extensions' not in gvars:
       gvars.update({ 'jinja2_extensions': [] })
 
-    else:
-      sys.path.append(os.getcwd())
-
     jinja2_options = {
       'undefined': jinja2.StrictUndefined,
       'trim_blocks': True,
@@ -467,6 +467,8 @@ class JinjaFx():
       'keep_trailing_newline': True
     }
 
+    sys.path += exts_dirs
+
     if isinstance(template, bytes) or isinstance(template, str):
       env = jinja2.Environment(extensions=gvars['jinja2_extensions'], **jinja2_options)
       [env.filters.update(f) for f in jinja2_filters]
@@ -485,6 +487,10 @@ class JinjaFx():
       'search': self.__jfx_search
     })
 
+    env.globals.update({
+      'lookup': self.__jfx_lookup
+    })
+
     env.globals.update({ 'jinjafx': {
       'version': __version__,
       'jinja2_version': jinja2.__version__,
@@ -497,6 +503,7 @@ class JinjaFx():
       'fields': self.__jfx_fields,
       'setg': self.__jfx_setg,
       'getg': self.__jfx_getg,
+      'now': self.__jfx_now,
 #      'nslookup': self.__jfx_nslookup,
       'rows': max([0, len(self.__g_datarows) - 1]),
       'data': [r[1:] if isinstance(r[0], int) else r for r in self.__g_datarows]
@@ -519,6 +526,9 @@ class JinjaFx():
         env.globals['jinjafx'].update({ 'row': 0 })
         self.__g_row = 0
 
+      self.__g_vars = gvars
+      self.__g_vars.update(rowdata)
+
       try:
         content = template.render(rowdata)
 
@@ -571,6 +581,21 @@ class JinjaFx():
     return outputs
 
 
+  def __jfx_lookup(self, method, variable, default=None):
+    if method == 'vars':
+      if variable in self.__g_vars:
+        return self.__g_vars[variable]
+
+      elif default is not None:
+        return default
+  
+      else:
+        raise jinja2.exceptions.UndefinedError('\'lookup\' variable \'' + variable + '\' is undefined')
+
+    else:
+      raise jinja2.exceptions.UndefinedError('\'lookup\' with method \'' + method + '\' is undefined')
+
+
   def __jfx_data_counter(self, m, orow, col, row):
     start = m.group(1)
     increment = m.group(2)
@@ -808,6 +833,16 @@ class JinjaFx():
     return self.__g_dict.get('_val_' + str(key), default)
 
 
+  def __jfx_now(self, fmt=None, tz='UTC'):
+    tz = pytz.timezone(tz)
+
+    if fmt is not None:
+      return datetime.datetime.utcnow().astimezone(tz).strftime(fmt)
+
+    else:
+      return str(datetime.datetime.utcnow().astimezone(tz))
+
+
 #  def __jfx_nslookup(self, v, family=46):
 #    try:
 #      if re.match(r'^(?:[0-9a-f:]+:+)+[0-9a-f]+$', v, re.I): # IPv6
